package com.muimi.mail;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import com.muimi.mail.model.Account;
import com.muimi.mail.model.AccountRoot;
import com.muimi.mail.model.LocalFolder;
import com.muimi.mail.ui.FolderView;
import com.muimi.mail.ui.MessageView;
import com.muimi.mail.ui.editor.MessageModel;

/**
 * [vOCNX
 * @author Shinji Miyamoto
 */
public class MailPlugin extends AbstractUIPlugin{

	/**
	 * bZ[Wr[ID
	 */
	public final static String MESSAGE_VIEW_ID = "com.muimi.mail.ui.MessageView";

	/**
	 * tH_r[ID
	 */
	public final static String FOLDER_VIEW_ID = "com.muimi.mail.ui.FolderView";

	/**
	 * [GfB^ID
	 */
	public final static String EDITOR_ID = "com.muimi.mail.ui.editor.MailEditor";

	/**
	 * ̃vOCsingletonCX^X
	 */
	private static MailPlugin plugin;

	/**
	 * AJEg[gCX^X
	 */
	private AccountRoot accountRoot;



	/**
	 * @param descriptor
	 */
	public MailPlugin(IPluginDescriptor descriptor) {
		super(descriptor);
		plugin = this;
		
		accountRoot = new AccountRoot();
		accountRoot.resotre();
	}
	
	
	public static MailPlugin getDefault(){
		return plugin;
	}



	public static IWorkbenchWindow getWindow(){
		return PlatformUI.getWorkbench().getActiveWorkbenchWindow();
	}	
	
	public static IWorkbenchPage getPage(){
		return getWindow().getActivePage();
	}
	
	public static Shell getShell(){
		return getWindow().getShell();
	}

	public static void showView(String viewId){
		try {
			getPage().showView(viewId);
		} catch (PartInitException e) {
			logError("fail to show view", e);
		}
	}
	



	public static void logError(String message, Throwable t) {
		if(t != null){
			t.printStackTrace();
		}
		//getDefault().getLog().log(new Status(IStatus.ERROR,getDefault().getDescriptor().getUniqueIdentifier(),IStatus.OK,message,t));
	}

	public static void logWarning(String message, Throwable t) {
		if(t != null){
			t.printStackTrace();
		}
		//getDefault().getLog().log(new Status(IStatus.ERROR,getDefault().getDescriptor().getUniqueIdentifier(),IStatus.OK,message,t));
	}

	public static void logInfo(String message, Throwable t) {
		t.printStackTrace();
		//getDefault().getLog().log(new Status(IStatus.INFO,getDefault().getDescriptor().getUniqueIdentifier(),IStatus.OK,message,t));
	}
	
	public static void showError(String message, Throwable t){
		MessageDialog.openError(getShell(), "error", message);
		logError(message, t);
	}
	
	
	
	

	public AccountRoot getAccountRoot(){
		return accountRoot;
	}
	
	

	
	
	public void createNewAccount(Account account){
		try {
			FolderView view = (FolderView)getPage().showView(FOLDER_VIEW_ID);
			view.addAccount(account);
		} catch (PartInitException e) {
			e.printStackTrace();
		}		
	}
	
	public void openLocalFolder(LocalFolder folder){
		try {
			MessageView view = (MessageView)getPage().showView(MESSAGE_VIEW_ID);
			view.setLocalFolder(folder);
		} catch (PartInitException e) {
			e.printStackTrace();
		}
	}
	
	public void openRemoteFolder(LocalFolder folder){
		try {
			MessageView view = (MessageView)getPage().showView(MESSAGE_VIEW_ID);
			view.setRemoteFolder(folder);
		} catch (PartInitException e) {
			e.printStackTrace();
		}
	}
	
	
	public void openMessage(MessageModel model){
		try {
			MailPlugin.getPage().openEditor(model , EDITOR_ID);
		} catch (PartInitException e) {
			logError("unexpected error", e);
		}
	}
	


	public void shutdown() throws CoreException {
		accountRoot.persist();
		PluginResources.freeResource();
		super.shutdown();
	}



	protected void initializeDefaultPluginPreferences() {
		super.initializeDefaultPluginPreferences();
		PluginPreference.initializeDefaultPluginPreferences();
	}

}
