package com.muimi.mail.ui;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferencePage;
import org.eclipse.jface.preference.PreferenceDialog;
import org.eclipse.jface.preference.PreferenceManager;
import org.eclipse.jface.preference.PreferenceNode;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.part.ViewPart;

import com.muimi.mail.MailPlugin;
import com.muimi.mail.PluginResources;
import com.muimi.mail.model.Account;
import com.muimi.mail.model.AccountRoot;
import com.muimi.mail.model.LocalFolder;
import com.muimi.mail.ui.editor.MessageModel;

/**
 * tH_r[
 * @author Shinji Miyamoto
 */
public class FolderView extends ViewPart {

	private TreeViewer treeViewer;



	public void createPartControl(Composite parent) {

		treeViewer = new TreeViewer(parent, SWT.NULL);
		treeViewer.setContentProvider(new TContentProvider());
		treeViewer.setLabelProvider(new TLabelProvider());

		MenuManager menuMgr = new MenuManager();
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new TMenuListener());
		Menu menu = menuMgr.createContextMenu(treeViewer.getControl());
		treeViewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, treeViewer);

		treeViewer.addDoubleClickListener(new TDoubleClickListener());

		treeViewer.setInput(MailPlugin.getDefault().getAccountRoot());
	}

	public void setFocus() {

	}



	class TContentProvider implements ITreeContentProvider {

		public Object[] getChildren(Object parent) {

			if (parent instanceof AccountRoot) {
				return ((AccountRoot) parent).getAcounts();
			}
			if (parent instanceof Account) {
				return ((Account) parent).getLocalFolders();
			}

			return new Object[0];
		}

		public Object getParent(Object element) {
			return null;
		}

		public boolean hasChildren(Object element) {
			return getChildren(element).length > 0;
		}

		public Object[] getElements(Object input) {
			if (input instanceof AccountRoot) {
				return ((AccountRoot) input).getAcounts();
			} else if (input instanceof Account) {
				return new Object[] { input };
			} else {
				return new Object[0];
			}

		}

		public void dispose() {
		}

		public void inputChanged(
			Viewer viewer,
			Object oldInput,
			Object newInput) {
		}
	}

	class TLabelProvider extends LabelProvider {

		public String getText(Object element) {
			if (element instanceof Account) {
				return ((Account) element).getLabel();
			} else if (element instanceof LocalFolder) {				
				return ((LocalFolder) element).getLabel();
			} else {
				return "";
			}
		}

		public Image getImage(Object element) {
			if (element instanceof Account) {
				return PluginResources.IMAGE_ACCOUNT.createImage();

			} else if (element instanceof LocalFolder) {
				LocalFolder folder = (LocalFolder)element;
				if(folder.isInBox()){
					return PluginResources.IMAGE_INBOX.createImage();					
				}else if(folder.isOutBox()){
					return PluginResources.IMAGE_OUTBOX.createImage();					
				}else if(folder.isTrash()){
					return PluginResources.IMAGE_TRASH.createImage();	
				}else{
					return PluginResources.IMAGE_FOLDER.createImage();
				}

			} else {
				return null;
			}
		}

	}

	class TDoubleClickListener implements IDoubleClickListener {

		public void doubleClick(DoubleClickEvent event) {
			IStructuredSelection ss =
				(IStructuredSelection) event.getSelection();
			if (!ss.isEmpty()) {
				Object element = ss.getFirstElement();
				if (element instanceof Account) {
					System.out.println("Account clicked");
				}else if(element instanceof LocalFolder){
					LocalFolder folder = (LocalFolder)element;
					MailPlugin.getDefault().openLocalFolder(folder);
				}
			}
		}
	}

	class TMenuListener implements IMenuListener {

		public void menuAboutToShow(IMenuManager manager) {

			if (isAccountSelected()) {
				manager.add(new CreateMailAction());
				manager.add(new Separator());

				manager.add(new DeleteAccountAction());
				manager.add(new UpdateAccountAction());
			
			}else if (isFolderSelected()){
				if(getSelectedFolder().isInBox()){
					manager.add(new OpenAndReceiveMailAction());
				}
			}
			

			manager.add(new CreateAccountAction());
			manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		}
	}
	
	
	
	
	private boolean isAccountSelected() {
		Object element =
			((StructuredSelection) treeViewer.getSelection()).getFirstElement();
		if (element instanceof Account) {
			return true;
		} else {
			return false;
		}
	}
	
	private boolean isFolderSelected(){
		Object element =
			((StructuredSelection) treeViewer.getSelection()).getFirstElement();
		if (element instanceof LocalFolder) {
			return true;
		} else {
			return false;
		}
	}
	
	
	public Account getSelectedAccount(){
		Object element =
			((StructuredSelection) treeViewer.getSelection()).getFirstElement();
		if (element instanceof Account) {
			return (Account)element;
		} else {
			throw new IllegalStateException("invalid");
		}		
	}
	
	public LocalFolder getSelectedFolder(){
		Object element =
			((StructuredSelection) treeViewer.getSelection()).getFirstElement();
		if (element instanceof LocalFolder) {
			return (LocalFolder)element;
		} else {
			throw new IllegalStateException("invalid");
		}	
	}
	
	
	
	public void addAccount(Account account){
		MailPlugin.getDefault().getAccountRoot().addAccount(account);
		treeViewer.refresh();
	}
	
	public void deleteAccount(Account account){
		MailPlugin.getDefault().getAccountRoot().removeAccount(account, true);
		treeViewer.refresh();
	}
	


	
	class CreateAccountAction extends Action{		
		public CreateAccountAction(){
			setText("AJEg쐬");
		}
		public void run() {
			CreateAccountWizard wizard = new CreateAccountWizard();
			//wizard.init(PlatformUI.getWorkbench(), null);
			WizardDialog dialog =
				new WizardDialog(MailPlugin.getShell(), wizard);
			dialog.open();
		}
	}

	class DeleteAccountAction extends Action{		
		public DeleteAccountAction(){
			setText("AJEg폜");
		}
		public void run() {			
			Account account = getSelectedAccount();

			boolean result = MessageDialog.openConfirm(MailPlugin.getShell(), 
				"delete account", 
				account.getLabel() + " 폜܂H" + "\n"
				+ "t@CVXẽfBNg " + account.getLocalStorePath() + " 폜܂B"
			);
			
			if(result){
				deleteAccount(account);
			}
		}
	}
	
	class UpdateAccountAction extends Action{		
		public UpdateAccountAction(){
			setText("AJEgύX");
		}
		public void run() {
			Account account = getSelectedAccount();
			
			PreferenceManager manager = new PreferenceManager();
			PreferenceDialog dialog = new PreferenceDialog(MailPlugin.getShell(), manager);

			IPreferencePage page = new UpdateAccountPropertyPage(account);
			PreferenceNode node = new PreferenceNode("page1", page);
			manager.addToRoot(node);

			dialog.create();
			dialog.getShell().setText("AJEgZbeBO");
			dialog.open();
		}
	}
	
//	class ReceiveMailAction extends Action{		
//		public ReceiveMailAction(){
//			setText("M");
//		}
//		public void run() {
//			Account account = getSelectedAccount();
//			try {
//				account.receiveMail();
//			} catch (MessagingException e) {
//				MailPlugin.logError("fail to receive mail", e);
//			}
//		}
//	}
	
	class OpenAndReceiveMailAction extends Action{		
		public OpenAndReceiveMailAction(){
			setText("M");
		}
		public void run() {
			LocalFolder folder = getSelectedFolder();
			MailPlugin.getDefault().openRemoteFolder(folder);
		}
	}
	
	class CreateMailAction extends Action{		
		public CreateMailAction(){
			setText("VK[");
		}
		public void run() {
			MessageModel messageModel = getSelectedAccount().createNewMessageModel();
			MailPlugin.getDefault().openMessage(messageModel);
		}
	}
	
}
