package com.muimi.mail.ui;

import java.io.UnsupportedEncodingException;
import java.util.Date;

import javax.mail.Flags;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.event.MessageCountEvent;
import javax.mail.event.MessageCountListener;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeUtility;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.part.ViewPart;

import com.muimi.mail.MailPlugin;
import com.muimi.mail.PluginResources;
import com.muimi.mail.model.LocalFolder;
import com.muimi.mail.ui.editor.MessageModel;

/**
 * bZ[Wr[B
 * e[ur[\̂
 * @author Shinji Miyamoto
 */
public class MessageView extends ViewPart {

	private TableColumn subjectColumn;
	private TableColumn fromColumn;
	private TableColumn dateColumn;
	private TableColumn sizeColumn;
	private TableColumn mailerColumn;


	private TableViewer viewer;
	private TContentProvider contentProvider;
	private TLabelProvier labelProvider;

	private TSorter sorter;

	private LocalFolder folder;



	public void createPartControl(Composite parent) {

		contentProvider = new TContentProvider();
		labelProvider = new TLabelProvier();

		viewer = new TableViewer(parent, SWT.FULL_SELECTION | SWT.BORDER);
		viewer.setContentProvider(contentProvider);
		viewer.setLabelProvider(labelProvider);


		createColumn("", 40, SWT.RIGHT);
		subjectColumn = createColumn("Subject", 100, SWT.RIGHT);
		fromColumn = createColumn("From", 160, SWT.LEFT);
		dateColumn = createColumn("Date", 160, SWT.LEFT);
		sizeColumn = createColumn("Size", 160, SWT.LEFT);
		mailerColumn = createColumn("X-Mailer", 160, SWT.LEFT);

		Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		viewer.addSelectionChangedListener(new TSelectionListener());
		viewer.addDoubleClickListener(new TDoubleClickListener());

		sorter = new TSorter();		
		viewer.setSorter(sorter);

		MenuManager menuMgr = new MenuManager();
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new TMenuListener());
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);


//		IActionBars actionBars = getViewSite().getActionBars();
//		IMenuManager ctxMenu = actionBars.getMenuManager();
//		ctxMenu.add(new ListAction());
//		ctxMenu.add(new NewMailAction());
	}


	private TableColumn createColumn(String text, int size, int style){
		TableColumn column = new TableColumn(viewer.getTable(), style);
		column.setText(text);
		column.setWidth(size);
		
		column.addSelectionListener(new HeaderSelectionListener());
		
		return column;
	}


	public void setLocalFolder(LocalFolder folder){
		closeIfOpen();
		this.folder = folder;
		try {
			Message[] messages = folder.getMessages();
			viewer.setInput(messages);
		} catch (MessagingException e) {
			MailPlugin.logError("fail to getMessage", e);
		}
	}

	public void setRemoteFolder(LocalFolder folder){
		closeIfOpen();
		this.folder = folder;
		try {
			Message[] messages = folder.getRemoteMessages();
			folder.addMessageCountListener(new MyMessageCountListener());
			viewer.setInput(messages);
		} catch (MessagingException e) {
			MailPlugin.logError("fail to getMessage", e);
		}
	}

	private void closeIfOpen(){
		if(folder != null){
			try {
				folder.close();
			} catch (MessagingException e) {
				// TODO ꂽ catch ubN
				e.printStackTrace();
			}
		}
	}



	public void setFocus() {
	}




	public Message getSelectedMessage(){
		StructuredSelection ss =(StructuredSelection)viewer.getSelection();
		if(!ss.isEmpty()){
			return (Message)ss.getFirstElement();
		}else{
			return null;
		}
	}



	class HeaderSelectionListener implements SelectionListener{

		public void widgetSelected(SelectionEvent e) {
			TableColumn column = (TableColumn)e.getSource();
			sorter.setSortProperty(column);
			viewer.refresh();
		}

		public void widgetDefaultSelected(SelectionEvent e) {
		}
	}


	class TSorter extends ViewerSorter{
		
		private TableColumn sortKey;
		private boolean desc;
		
		public void setSortProperty(TableColumn newKey){
			if(sortKey == newKey){
				desc = !desc;
			}else{
				sortKey = newKey;
				desc = false;
			}
		}
		
		public int compare(Viewer viewer, Object e1, Object e2) {
			Message m1 = (Message)e1;
			Message m2 = (Message)e2;

			int result = 0;

			try {
				if(sortKey == sizeColumn){
					result = compareBySize(m1, m2);
				}else if(sortKey == dateColumn){
					result = compareByDate(m1, m2);
				}else if(sortKey == subjectColumn){
					result = compareBySubject(m1, m2);
				}else if(sortKey == fromColumn){
					result = compareByDate(m1, m2);
				}else if(sortKey == mailerColumn){
					result = compareByMailer(m1, m2);
				}

			} catch (MessagingException e) {
				MailPlugin.logError("error in sort", e);
			}			
			
			
			if(!desc){
				result =  0 - result;
			}
			
			return result;
		}
		

		private int compareBySubject(Message m1, Message m2) throws MessagingException{
			String s1 = m1.getSubject();
			String s2 = m2.getSubject();
			
			return compareString(s1, s2);
		}


		private int compareByFrom(Message m1, Message m2) throws MessagingException{
			String s1;
			String s2;
			try {
				//TODO too muda muda...
				s1 = getFromString(m1);
				s2 = getFromString(m2);
			} catch (UnsupportedEncodingException e) {
				MailPlugin.logError("fail to read from", e);
				return 0;
			}

			return compareString(s1, s2);
		}

		private int compareBySize(Message m1, Message m2) throws MessagingException{
			int size1 = m1.getSize();
			int size2 = m2.getSize();
					
			if(size1 > size2){
				return 1;
			}else if(size1 < size2){
				return -1;
			}else{
				return 0;
			}
		}

		private int compareByDate(Message m1, Message m2) throws MessagingException{
			Date date1 = m1.getSentDate();
			Date date2 = m2.getSentDate();

			if(date1 == date2){
				return 0;
			}else if(date1 == null){
				return 1;
			}else if(date2 == null){
				return -1;
			}else{
				return date1.compareTo(date2);
			}
		}
		
		private int compareByMailer(Message m1, Message m2) throws MessagingException{
			String s1 = getMailerString(m1);
			String s2 = getMailerString(m2);
			return compareString(s1, s2);
		}
		
		private int compareString(String s1, String s2){
			if(s1 == s2){
				return 0;
			}else if(s1 == null){
				return 1;
			}else if(s2 == null){
				return -1;
			}else{
				return s1.compareTo(s2);
			}
		}
	}


	class TSelectionListener implements ISelectionChangedListener{
		public void selectionChanged(SelectionChangedEvent event) {
			//System.out.println(event.getSource());
		}
	}

	
	class TDoubleClickListener implements IDoubleClickListener{

		public void doubleClick(DoubleClickEvent event) {
			IStructuredSelection ss = (IStructuredSelection)event.getSelection();
			if(!ss.isEmpty()) {
				Message message = (Message)ss.getFirstElement();
				try {
					MessageModel model = new MessageModel(folder.getAccount(), message);
					MailPlugin.getDefault().openMessage(model);
				} catch (Exception e) {
					MailPlugin.logError("fail to create message", e);
				}
			}
		}
	}


	class TContentProvider implements IStructuredContentProvider{

		public Object[] getElements(Object inputElement) {
			return (Message[])inputElement;
		}

		public void dispose() {
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}



	class MyMessageCountListener implements MessageCountListener{

		public void messagesAdded(MessageCountEvent event) {
			System.out.println("add");
			Display.getDefault().asyncExec(new AddMessageTask(event.getMessages()));
		}

		public void messagesRemoved(MessageCountEvent event) {
		}
		
		class AddMessageTask extends Thread{
			private Message[] messages;
			public AddMessageTask(Message[] messages){
				this.messages = messages;
			}
			public void run(){
				//TODO This method should be called (by the content provider)
				viewer.add(messages);
			}
		}
		
	}


	class TLabelProvier implements ITableLabelProvider{

		public Image getColumnImage(Object element, int columnIndex) {
			Message message = (Message)element;
			try{
				switch(columnIndex){
					case 0:
						if(message.isMimeType("multipart/*")){
							return PluginResources.IMAGE_ATTACH.createImage();
						}else{
							return null;
						}
					default:
				}
			}catch(MessagingException e){
				MailPlugin.logError("error in get image", e);
			}
			return null;
		}

		public String getColumnText(Object element, int columnIndex) {
			Message message = (Message)element;
			try{
				switch(columnIndex){
					case 0:
//						if(message.isMimeType("multipart/*")){
//							return "*";
//						}else{
//							return "";
//						}
						Flags flags = message.getFlags();
						if(flags.contains(Flags.Flag.RECENT)){
							return "*";
						}else{
							return "";
						}
					case 1:
						return message.getSubject();
					case 2:
						return getFromString(message);
					case 3:
						Date date = message.getSentDate();
						return date != null ? date.toString() : "";

					case 4:
						return String.valueOf(message.getSize());

					case 5:
						return getMailerString(message);
					default:
						return "";
				}
			}catch(Exception e){
				MailPlugin.logError("error in get text", e);
				return "";
			}
		}

		public void addListener(ILabelProviderListener listener) {
		}

		public void dispose() {
		}

		public boolean isLabelProperty(Object element, String property) {
			return false;
		}

		public void removeListener(ILabelProviderListener listener) {
		}
	}
	
	
	
	class TMenuListener implements IMenuListener {

		public void menuAboutToShow(IMenuManager manager) {
			manager.add(new DeleteMessageAction());
			manager.add(new ReceiveMailAction());

			manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		}
	}
	
	
	class DeleteMessageAction extends Action{
		public DeleteMessageAction(){
			setText("폜");
		}
		public void run(){
			Message message = getSelectedMessage();
			try {
				folder.deleteMessage(message);
				viewer.remove(message);
			} catch (MessagingException e) {
				MailPlugin.logError("fail to mark delete",e);
			}
		}
	}
	
	class ReceiveMailAction extends Action{
		public ReceiveMailAction(){
			setText("ԐM");
		}
		public void run(){

		}
	}
	
	
	
	private String getFromString(Message message) throws UnsupportedEncodingException, MessagingException{	
		return MimeUtility.decodeText(InternetAddress.toString(message.getFrom()));
	}

	private String getMailerString(Message message) throws MessagingException{
		String[] mailer = message.getHeader("X-Mailer");
		if(mailer == null){
			return "";
		}else{
			StringBuffer sb = new StringBuffer();
			for(int i=0; i<mailer.length; i++){
				sb.append(mailer[i]);
				sb.append(",");
			}
			return sb.toString();
		}
	}
	

	public void dispose() {
		super.dispose();
	}

}
