package com.muimi.mail.ui.editor;


import javax.mail.MessagingException;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import com.muimi.mail.MailPlugin;
import com.muimi.mail.PluginResources;
import com.muimi.mail.ui.editor.content.ContentEditor_Old;
import com.muimi.mail.ui.editor.content.IContentEditor;



/**
 * [GfB^
 * @author Shinji Miyamoto
 */
public class MailEditor extends EditorPart  {

	private Text fromText;
	private Text toText;
	private Text subjectText;

	private IContentEditor contentEditor;

	private Composite attachCtrl;
	
	private boolean readOnly;
	
	private Composite parent;
	private Composite c;


	public void doSave(IProgressMonitor monitor) {
	}

	public void doSaveAs() {
	}

	public void gotoMarker(IMarker marker) {
	}

	public void init(IEditorSite site, IEditorInput input) throws PartInitException {		
		System.out.println("init");
		setSite(site);
		setInput(input);

		MessageModel message = getMessage();

		if(message.isReplyMessage()){
			setTitle("ԐM " + message.getSubject());
			readOnly = false;
		}else if(message.isNewMessage()){
			setTitle("VK");
			readOnly = false;
		}else{
			setTitle(message.getSubject());
			readOnly = true;
		}
	}


	public boolean isDirty() {
		return false;
	}

	public boolean isSaveAsAllowed() {
		return false;
	}


	public void createPartControl(Composite parent) {		

		this.parent = parent;

		MessageModel message = getMessage();
		

		c =  new Composite(parent, SWT.NULL);
		c.setLayout(new GridLayout(2, false));
//		c.setBackground(PluginResources.COL_BG);

		
		//head		
		Label fromLabel = createLabel(c, "From");		
		fromText = createTextField(c);
		setTextIfNotNull(fromText, message.getFrom());


		Label toLabel = createLabel(c, "To");
		toText = createTextField(c);
		setTextIfNotNull(toText, message.getTo());


		Label subjectLabel = createLabel(c, "Subject");
		subjectText = createTextField(c);		
		setTextIfNotNull(subjectText, message.getSubject());


		//content
		contentEditor = new ContentEditor_Old(getMessage());
		Control content = contentEditor.createContent(c, readOnly);
		contentEditor.setText(message.getContentText());;
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = 2;
		content.setLayoutData(gd);

	

		//attach
		if(message.isAttachmentExist()){

			createAttachCtrl();

			PortableFile[] attachs = message.getAttachments();
			for(int i=0; i<attachs.length; i++){
				PortableFile attach = attachs[i];
				AttachLabel label = new AttachLabel(attachCtrl, SWT.NULL);
				label.setFile(attach);
			}
		}


		createContextMenu(c);
	}

	
	
	private Label createLabel(Composite c, String text){
		Label label = new Label(c, SWT.NULL);
		label.setText(text);
//		label.setBackground(PluginResources.COL_BG);
		return label;
	}
	
	private Text createTextField(Composite c){
		
		Text text;
		
		if(readOnly){
			text = new Text(c, SWT.READ_ONLY);
		}else{
			text = new Text(c, SWT.BORDER);
		}
		
		text.setLayoutData(gd1());

		return text;
	}
	
	
//	private int def(){
//		if(readOnly){
//			return SWT.READ_ONLY;
//		}else{
//			return SWT.NULL;
//		}
//	}


	private GridData gd1(){
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 200;
		return gd;
	}


	private void setTextIfNotNull(Text widget, String value){
		if(value != null){
			widget.setText(value);
		}
	}
	private void setTextIfNotNull(StyledText widget, String value){
		if(value != null){
			widget.setText(value);
		}
	}



	private void createAttachCtrl(){
		Label attachLabel = new Label(c, SWT.NULL);
		attachLabel.setText("attach");
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		attachLabel.setLayoutData(gd);	
			
		attachCtrl = new Composite(c, SWT.BORDER);
			
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		attachCtrl.setLayoutData(gd);			
		attachCtrl.setLayout(new RowLayout());
		attachCtrl.setBackground(PluginResources.COL_BG);		
	}



	/**
	 * M܂B
	 */
	public void sendMail(){
		
		MessageModel model = getMessage();
		model.setTo(toText.getText());
		model.setFrom(fromText.getText());
		model.setSubject(subjectText.getText());
		model.setContentText(contentEditor.getText());

		try {
			model.sendMail();
		} catch (MessagingException e) {
			e.printStackTrace();
		}
		
		MailPlugin.getPage().closeEditor(this, false);
	}


	/**
	 * Ytt@Cǉ܂B
	 * @param file
	 */
	public void addAttach(PortableFile attach){
		if(attachCtrl == null){
			createAttachCtrl();
		}

		getMessage().addAttach(attach);
		AttachLabel label = new AttachLabel(attachCtrl, SWT.NULL);
		label.setFile(attach);
		
		System.out.println(attachCtrl.getChildren().length);
		
		//TODO Qȏ͂ŃtbVȂH
		c.layout(true);
		c.redraw();
	}
	
	
	
	
	public MessageModel getMessage(){
		return (MessageModel)getEditorInput();
	}
	
	
	
	
	



	private void createContextMenu(Composite c) {
	  MenuManager menuMgr =
		new MenuManager(getEditorSite().getId(), getSite().getId());
	  menuMgr.setRemoveAllWhenShown(true);

	  menuMgr.addMenuListener(new IMenuListener() {
		public void menuAboutToShow(IMenuManager manager) {
		  MailEditor.this.fillContextMenu(manager);
		}
	  });

	  Menu contextMenu = menuMgr.createContextMenu(c);
	  c.setMenu(contextMenu);
//	  table.setMenu(contextMenu);

	  getSite().registerContextMenu(
		menuMgr,
		getSite().getSelectionProvider());
	}

	private void fillContextMenu(IMenuManager menuMgr) {

		menuMgr.add(new SendAction());

		menuMgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));

	}

	class SendAction extends Action{
		public SendAction(){
			setText("send");
		}
	}





	public void setFocus() {
	}



}
