package com.muimi.mail.ui.editor;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeUtility;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;

import com.muimi.mail.MailPlugin;
import com.muimi.mail.model.Account;


/**
 * [bZ[W̃A_v^[B
 * EditorInputłB
 * @author Shinji Miyamoto
 */
public class MessageModel implements IEditorInput {

	private Account account;

	private Message message;

	private boolean newMessage = true;
	private boolean replayMessage = false;
	private String subject = "";
	private String to = "";
	private String from = "";
	private String contentText;
	private Date sentDate;
	
	private List attachments = new ArrayList();






	public MessageModel(Account account) {
		this.account = account;
	}


	public MessageModel(Account account, Message message)
		throws IOException, MessagingException {
		this(account);
	
		this.message = message;
		newMessage = false;

		subject = message.getSubject();

		to = InternetAddress.toString(
				message.getRecipients(Message.RecipientType.TO));
		if(to != null){
			to = MimeUtility.decodeText(to);
		}else{
			to = "";
		}

		from = InternetAddress.toString(message.getFrom());
		if(from != null){
			from = MimeUtility.decodeText(from);
		}else{
			from = "";
		}

		sentDate = message.getSentDate();


		extractContets(message);
		if(contentText == null){
			contentText = "";
		}
	}


	



	private void extractContets(Message message) throws MessagingException, IOException{

		try{
			if (message.isMimeType("text/plain")
				|| message.isMimeType("message/rfc822")) {
	
				contentText = message.getContent().toString();

			}else if(message.isMimeType("text/html")){
				contentText = message.getContent().toString();
				PortableFile file = new PortableFile(message);
				attachments.add(file);
				
			}else if(message.isMimeType("multipart/*")){
				extractMultiContent(message);

			}else{
				PortableFile file = new PortableFile(message);
				attachments.add(file);
			}
		
		}catch(Exception e){
			e.printStackTrace();
		}
	}


	private void extractMultiContent(Part message) throws MessagingException, IOException{
		System.out.println(message.getContentType());
		
		if (message.isMimeType("text/plain")
			|| message.isMimeType("message/rfc822")) {
			
			if(contentText == null){
				contentText = message.getContent().toString();
			}else{
				PortableFile file = new PortableFile(message);
				attachments.add(file);
			}
		
		} else if (message.isMimeType("multipart/*")) { // }`p[g̏ꍇ
			Multipart mp = (Multipart) message.getContent();
			int count = mp.getCount();
			for (int i = 0; i < count; i++) {
				extractMultiContent(mp.getBodyPart(i));
			}

		} else if (message.isMimeType("text/html")) { // HTML̏ꍇ

			PortableFile file = new PortableFile(message);
			attachments.add(file);

		} else { // ̑̏ꍇ

			PortableFile file = new PortableFile(message);
			attachments.add(file);
		}
	}





	public MessageModel createReply() {
		try{
			Message reply = message.reply(false);
			reply.setFrom(account.getUserInetAddress());
			reply.setText(getReplyConentText());
			MessageModel replyModel = new MessageModel(account , reply);
			replyModel.newMessage = true;
			replyModel.replayMessage = true;
			return replyModel;
		}catch(Exception e){
			MailPlugin.logError("unexpected error", e);
			throw new IllegalStateException();
		}
	}


	private String getReplyConentText(){
		String text = getContentText();
		BufferedReader r = new BufferedReader(new StringReader(text));
		StringBuffer sb = new StringBuffer();
		//regex /^/>/ 
		try {
			String line;
			while((line = r.readLine()) != null){
				sb.append(">" + line + "\n");
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return sb.toString();
	}


	public MessageModel createMessageAsContext(String to) {
		try{
			MimeMessage message = new MimeMessage(account.getRemoteSession());
			message.setFrom(account.getUserInetAddress());
			message.setSubject("VK");
			message.setText("");
			message.setRecipients(Message.RecipientType.TO, to);
			MessageModel messageModel = new MessageModel(account , message);
			messageModel.newMessage = true;
			return messageModel;
		}catch(Exception e){
			MailPlugin.logError("unexpected error", e);
			throw new IllegalStateException();
		}
	}




	public void sendMail() throws MessagingException {
		account.sendMail(this);
	}


	public boolean isAttachmentExist(){
		return !attachments.isEmpty();
	}
	
	
	public void addAttach(PortableFile attach){
		attachments.add(attach);
	}
	


//	public String[] getAttachmentNames(){
//		String[] filenames = new String[attachments.size()];
//		for(int i=0; i<attachments.size(); i++){
//			filenames[i] = ((PortableFile)attachments.get(i)).getName();
//		}
//		return filenames;
//	}

	public PortableFile[] getAttachments(){
		return (PortableFile[])attachments.toArray(new PortableFile[0]);
	}






	//method of IEditorInput
	public boolean exists() {
		return false;
	}

	public ImageDescriptor getImageDescriptor() {
		return null;
	}

	public String getName() {
		return "hoge";
	}

	public IPersistableElement getPersistable() {
		return null;
	}

	public String getToolTipText() {
		return "tooltip hoge";
	}

	public Object getAdapter(Class adapter) {
		return null;
	}

	/**
	 * @return
	 */
	public String getContentText() {
		return contentText;
	}

	/**
	 * @return
	 */
	public String getFrom() {
		return from;
	}

	/**
	 * @return
	 */
	public boolean isNewMessage() {
		return newMessage;
	}

	public boolean isReplyMessage(){
		return replayMessage;
	}

	/**
	 * @return
	 */
	public String getSubject() {
		return subject;
	}

	/**
	 * @return
	 */
	public String getTo() {
		return to;
	}

	/**
	 * @param string
	 */
	public void setContentText(String string) {
		contentText = string;
	}

	/**
	 * @param string
	 */
	public void setFrom(String string) {
		from = string;
	}

	/**
	 * @param b
	 */
	public void setNewMessage(boolean b) {
		newMessage = b;
	}

	/**
	 * @param string
	 */
	public void setSubject(String string) {
		subject = string;
	}

	/**
	 * @param string
	 */
	public void setTo(String string) {
		to = string;
	}

	/**
	 * @return
	 */
	public Date getSentDate() {
		return sentDate;
	}

}
